@@ -84,7 +84,8 @@ context 'API' do
       assert_match(/reader\.rb.*prepare_lines/, exception.backtrace[0..4].join(?\n))
     end
 
-    test 'should convert filename that contains non-ASCII characters independent of default encodings' do
+    # NOTE JRuby for Windows does not permit creating a file with non-Windows-1252 characters in the filename
+    test 'should convert filename that contains non-ASCII characters independent of default encodings', unless: (jruby? && windows?) do
       old_external = Encoding.default_external
       old_internal = Encoding.default_internal
       old_verbose = $VERBOSE
@@ -519,6 +520,43 @@ context 'API' do
       assert_equal 96, unordered_complex_items[4].lineno
     end
 
+    # FIXME see #3966
+    test 'should assign incorrect lineno for single-line paragraph inside a conditional preprocessor directive' do
+      input = <<~'EOS'
+      :conditional-attribute:
+
+      before
+
+      ifdef::conditional-attribute[]
+      subject
+      endif::[]
+
+      after
+      EOS
+
+      doc = document_from_string input, sourcemap: true
+      # FIXME the second line number should be 6 instead of 7
+      assert_equal [3, 7, 9], (doc.find_by context: :paragraph).map(&:lineno)
+    end
+
+    test 'should assign correct lineno for multi-line paragraph inside a conditional preprocessor directive' do
+      input = <<~'EOS'
+      :conditional-attribute:
+
+      before
+
+      ifdef::conditional-attribute[]
+      subject
+      subject
+      endif::[]
+
+      after
+      EOS
+
+      doc = document_from_string input, sourcemap: true
+      assert_equal [3, 6, 10], (doc.find_by context: :paragraph).map(&:lineno)
+    end
+
     # NOTE this does not work for a list continuation that attached to a grandparent
     test 'should assign correct source location to blocks that follow a detached list continuation' do
       input = <<~'EOS'
