@@ -262,6 +262,14 @@ context 'AttributeList' do
     assert_equal expected, attributes
   end
 
+  test 'should not assign nil to attribute mapped to missing positional attribute' do
+    attributes = {}
+    line = 'alt text,,100'
+    expected = { 1 => 'alt text', 2 => nil, 3 => '100', 'alt' => 'alt text', 'height' => '100' }
+    Asciidoctor::AttributeList.new(line).parse_into(attributes, %w(alt width height))
+    assert_equal expected, attributes
+  end
+
   test 'rekey positional attributes' do
     attributes = { 1 => 'source', 2 => 'java' }
     expected = { 1 => 'source', 2 => 'java', 'style' => 'source', 'language' => 'java' }
