@@ -634,6 +634,18 @@ context 'Document' do
       assert_xpath '//*[@id="preamble"]//p[text()="Document Title"]', doc.convert, 1
     end
 
+    test 'document header can reference intrinsic doctitle attribute' do
+      input = <<~'EOS'
+      = ACME Documentation
+      :intro: Welcome to the {doctitle}!
+
+      {intro}
+      EOS
+      doc = document_from_string input
+      assert_equal 'Welcome to the ACME Documentation!', (doc.attr 'intro')
+      assert_xpath '//p[text()="Welcome to the ACME Documentation!"]', doc.convert, 1
+    end
+
     test 'document with title attribute entry overrides doctitle attribute entry' do
       input = <<~'EOS'
       = Document Title
@@ -691,20 +703,87 @@ context 'Document' do
       assert_xpath '//*[@id="preamble"]//p[text()="Override"]', doc.convert, 1
     end
 
-    test 'header substitutions should be applied to the value of the doctitle attribute' do
+    test 'should apply header substitutions to value of the doctitle attribute assigned from implicit doctitle' do
       input = <<~'EOS'
-      = <Foo> & <Bar>
+      = <Foo> {plus} <Bar>
 
       The name of the game is {doctitle}.
       EOS
 
       doc = document_from_string input
-      assert_equal '&lt;Foo&gt; &amp; &lt;Bar&gt;', (doc.attr 'doctitle')
-      assert_includes doc.blocks[0].content, '&lt;Foo&gt; &amp; &lt;Bar&gt;'
+      assert_equal '&lt;Foo&gt; &#43; &lt;Bar&gt;', (doc.attr 'doctitle')
+      assert_includes doc.blocks[0].content, '&lt;Foo&gt; &#43; &lt;Bar&gt;'
+    end
+
+    test 'should substitute attribute reference in implicit document title for attribute defined earlier in header' do
+      using_memory_logger do |logger|
+        input = <<~'EOS'
+        :project-name: ACME
+        = {project-name} Docs
+
+        {doctitle}
+        EOS
+        doc = document_from_string input, attributes: { 'attribute-missing' => 'warn' }
+        assert_empty logger
+        assert_equal 'ACME Docs', (doc.attr 'doctitle')
+        assert_equal 'ACME Docs', doc.doctitle
+        assert_xpath '//p[text()="ACME Docs"]', doc.convert, 1
+      end
+    end
+
+    test 'should not warn if implicit document title contains attribute reference for attribute defined later in header' do
+      using_memory_logger do |logger|
+        input = <<~'EOS'
+        = {project-name} Docs
+        :project-name: ACME
+
+        {doctitle}
+        EOS
+        doc = document_from_string input, attributes: { 'attribute-missing' => 'warn' }
+        assert_empty logger
+        assert_equal '{project-name} Docs', (doc.attr 'doctitle')
+        assert_equal 'ACME Docs', doc.doctitle
+        assert_xpath '//p[text()="{project-name} Docs"]', doc.convert, 1
+      end
     end
 
     test 'should recognize document title when preceded by blank lines' do
       input = <<~'EOS'
+
+      = Title
+
+      preamble
+
+      == Section 1
+
+      text
+      EOS
+      output = convert_string input, safe: Asciidoctor::SafeMode::SAFE
+      assert_css '#header h1', output, 1
+      assert_css '#content h1', output, 0
+    end
+
+    test 'should recognize document title when preceded by blank lines introduced by a preprocessor conditional' do
+      input = <<~'EOS'
+      ifdef::sectids[]
+
+      :foo: bar
+      endif::[]
+      = Title
+
+      preamble
+
+      == Section 1
+
+      text
+      EOS
+      output = convert_string input, safe: Asciidoctor::SafeMode::SAFE
+      assert_css '#header h1', output, 1
+      assert_css '#content h1', output, 0
+    end
+
+    test 'should recognize document title when preceded by blank lines after an attribute entry' do
+      input = <<~'EOS'
       :doctype: book
 
       = Title
@@ -1839,6 +1918,25 @@ context 'Document' do
       assert_xpath '/h2[text()="NAME"]/following-sibling::*[@class="sectionbody"]', output, 1
       assert_xpath '/h2[text()="NAME"]/following-sibling::*[@class="sectionbody"]/p[text()="asciidoctor - converts AsciiDoc source files to HTML, DocBook and other formats"]', output, 1
     end
+
+    test 'should output all mannames in name section in man page output' do
+      input = <<~'EOS'
+      = eve(1)
+      :doctype: manpage
+
+      == NAME
+
+      eve, probe - analyzes an image to determine if it is a picture of a life form
+
+      == SYNOPSIS
+
+      *eve* [OPTION]... FILE...
+      EOS
+
+      output = convert_string input
+      assert_css 'body.manpage', output, 1
+      assert_xpath '//h2[text()="NAME"]/following-sibling::*[@class="sectionbody"]/p[text()="eve, probe - analyzes an image to determine if it is a picture of a life form"]', output, 1
+    end
   end
 
   context 'Secure Asset Path' do
