@@ -1412,7 +1412,7 @@ context 'Extensions' do
       end
     end
 
-    test 'should apply specified subs to inline node returned by process method' do
+    test 'should apply subs specified as symbol to inline node returned by process method' do
       begin
         Asciidoctor::Extensions.register do
           inline_macro do
@@ -1430,6 +1430,42 @@ context 'Extensions' do
       end
     end
 
+    test 'should apply subs specified as array to inline node returned by process method' do
+      begin
+        Asciidoctor::Extensions.register do
+          inline_macro do
+            named :say
+            process do |parent, target, attrs|
+              create_inline_pass parent, %(*#{target}*), attributes: { 'subs' => [:specialchars, :quotes] }
+            end
+          end
+        end
+
+        output = convert_string_to_embedded 'say:{lt}message{gt}[]', doctype: :inline
+        assert_equal '<strong>&lt;message&gt;</strong>', output
+      ensure
+        Asciidoctor::Extensions.unregister_all
+      end
+    end
+
+    test 'should apply subs specified as string to inline node returned by process method' do
+      begin
+        Asciidoctor::Extensions.register do
+          inline_macro do
+            named :say
+            process do |parent, target, attrs|
+              create_inline_pass parent, %(*#{target}*), attributes: { 'subs' => 'specialchars,quotes' }
+            end
+          end
+        end
+
+        output = convert_string_to_embedded 'say:{lt}message{gt}[]', doctype: :inline
+        assert_equal '<strong>&lt;message&gt;</strong>', output
+      ensure
+        Asciidoctor::Extensions.unregister_all
+      end
+    end
+
     test 'should prefer attributes parsed from inline macro over default attributes' do
       begin
         Asciidoctor::Extensions.register do
