@@ -454,7 +454,7 @@ class ReaderTest < Minitest::Test
         data.push ''
         doc = Asciidoctor::Document.new data
         reader = doc.reader
-        assert_equal SAMPLE_DATA, reader.lines
+        assert_equal [''] + SAMPLE_DATA, reader.lines
       end
 
       test 'should prepare and normalize lines from String data' do
@@ -464,7 +464,14 @@ class ReaderTest < Minitest::Test
         data_as_string = data * ::Asciidoctor::LF
         doc = Asciidoctor::Document.new data_as_string
         reader = doc.reader
-        assert_equal SAMPLE_DATA, reader.lines
+        assert_equal [''] + SAMPLE_DATA, reader.lines
+      end
+
+      test 'should drop all lines if all lines are empty' do
+        data = ['', ' ', '', ' ']
+        doc = Asciidoctor::Document.new data
+        reader = doc.reader
+        assert reader.lines.empty?
       end
 
       test 'should clean CRLF from end of lines' do
@@ -854,7 +861,7 @@ class ReaderTest < Minitest::Test
         end
       end
 
-      test 'unreadable file referenced by include directive is replaced by warning', unless: windows? do
+      test 'unreadable file referenced by include directive is replaced by warning', unless: (windows? || Process.euid == 0) do
         include_file = File.join DIRNAME, 'fixtures', 'chapter-a.adoc'
         FileUtils.chmod 0000, include_file
         input = <<~'EOS'
@@ -1238,6 +1245,31 @@ class ReaderTest < Minitest::Test
         assert_includes output, expected
       end
 
+      test 'should recognize tag wildcard if not at head of list' do
+        input = <<~'EOS'
+        ----
+        include::fixtures/tagged-class.rb[tags=init;**;*;!bark-other]
+        ----
+        EOS
+
+        output = convert_string_to_embedded input, safe: :safe, base_dir: DIRNAME
+        # NOTE cannot use single-quoted heredoc because of https://github.com/jruby/jruby/issues/4260
+        expected = <<~EOS.chop
+        class Dog
+          def initialize breed
+            @breed = breed
+          end
+
+          def bark
+            if @breed == 'beagle'
+              'woof woof woof woof woof'
+            end
+          end
+        end
+        EOS
+        assert_includes output, expected
+      end
+
       test 'include directive selects lines for all tags when value of tags attribute is wildcard' do
         input = <<~'EOS'
         ----
@@ -1289,7 +1321,7 @@ class ReaderTest < Minitest::Test
         assert_includes output, expected
       end
 
-      test 'include directive skips lines all tagged lines when value of tags attribute is negated wildcard' do
+      test 'include directive skips lines all tagged regions when value of tags attribute is negated wildcard' do
         input = <<~'EOS'
         ----
         include::fixtures/tagged-class.rb[tags=!*]
@@ -1301,6 +1333,40 @@ class ReaderTest < Minitest::Test
         assert_includes output, expected
       end
 
+      test 'include directive skips lines all tagged regions except ones enabled when value of tags attribute is negated wildcard' do
+        input = <<~'EOS'
+        ----
+        include::fixtures/tagged-class.rb[tags=!*;init]
+        ----
+        EOS
+
+        output = convert_string_to_embedded input, safe: :safe, base_dir: DIRNAME
+        # NOTE cannot use single-quoted heredoc because of https://github.com/jruby/jruby/issues/4260
+        expected = <<~EOS.chop
+        class Dog
+          def initialize breed
+            @breed = breed
+          end
+        end
+        EOS
+        assert_includes output, expected
+      end
+
+      test 'include directive does not include tag that has been included then excluded' do
+        input = <<~'EOS'
+        ----
+        include::fixtures/tagged-class.rb[tags=!*;init;!init]
+        ----
+        EOS
+
+        output = convert_string_to_embedded input, safe: :safe, base_dir: DIRNAME
+        expected = <<~'EOS'.chop
+        class Dog
+        end
+        EOS
+        assert_includes output, expected
+      end
+
       test 'include directive selects specified tagged lines and ignores the other tag directives' do
         input = <<~'EOS'
         [indent=0]
